@@ -3,10 +3,7 @@ use core::fmt::{self, Debug};
 use core::iter::FromIterator;
 use core::ops::Index;
 
-use crate::tree::{ConsumingIter, Iter, IterMut, SGTree};
-
-#[cfg(feature = "high_assurance")]
-use crate::tree::SGErr;
+use crate::tree::{ConsumingIter, Iter, IterMut, SGErr, SGTree};
 
 /// Ordered map.
 /// A wrapper interface for `SGTree`.
@@ -33,6 +30,32 @@ impl<K: Ord, V> SGMap<K, V> {
         SGMap { bst: SGTree::new() }
     }
 
+    /// The [original scapegoat tree paper's](https://people.csail.mit.edu/rivest/pubs/GR93.pdf) alpha, `a`, can be chosen in the range `0.5 <= a < 1.0`.
+    /// `a` tunes how "aggressively" the data structure self-balances.
+    /// It controls the trade-off between total rebuilding time and maximum height guarantees.
+    ///
+    /// * As `a` approaches `0.5`, the tree will rebalance more often. Ths means slower insertions, but faster lookups and deletions.
+    ///     * An `a` equal to `0.5` means a tree that always maintains a perfect balance (e.g."complete" binary tree, at all times).
+    ///
+    /// * As `a` approaches `1.0`, the tree will rebalance less often. This means quicker insertions, but slower lookups and deletions.
+    ///     * If `a` reached `1.0`, it'd mean a tree that never rebalances.
+    ///
+    /// Returns `Err` if `0.5 <= alpha_num / alpha_denom < 1.0` isn't `true` (invalid `a`, out of range).
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scapegoat::SGMap;
+    ///
+    /// let mut map: SGMap<isize, isize> = SGMap::new();
+    ///
+    /// // Set 2/3, e.g. `a = 0.666...` (it's default value).
+    /// assert!(map.set_rebal_param(2.0, 3.0).is_ok());
+    /// ```
+    pub fn set_rebal_param(&mut self, alpha_num: f32, alpha_denom: f32) -> Result<(), SGErr> {
+        self.bst.set_rebal_param(alpha_num, alpha_denom)
+    }
+
     /// `#![no_std]`: total capacity, e.g. maximum number of map pairs.
     /// Attempting to insert pairs beyond capacity will panic, unless the `high_assurance` feature is enabled.
     ///
@@ -634,7 +657,7 @@ where
 // From array.
 impl<K, V, const N: usize> From<[(K, V); N]> for SGMap<K, V>
 where
-    K: Ord
+    K: Ord,
 {
     /// ```
     /// use scapegoat::SGMap;
@@ -669,7 +692,7 @@ where
 // Construct from iterator.
 impl<K, V> FromIterator<(K, V)> for SGMap<K, V>
 where
-    K: Ord
+    K: Ord,
 {
     fn from_iter<I: IntoIterator<Item = (K, V)>>(iter: I) -> Self {
         let mut sgm = SGMap::new();
@@ -681,7 +704,7 @@ where
 // Extension from iterator.
 impl<K, V> Extend<(K, V)> for SGMap<K, V>
 where
-    K: Ord
+    K: Ord,
 {
     fn extend<T: IntoIterator<Item = (K, V)>>(&mut self, iter: T) {
         self.bst.extend(iter);
